﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;

namespace ColumnAndLineChart
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            QuaterData = new ObservableCollection<Data>();
            YearData = new ObservableCollection<Data>();
            Random r = new Random();

            for (int i = 0; i < 12; i++)
            {
                QuaterData.Add(new Data
                {
                    Label = "200" + (i/4) + "Q" + ((i % 4) + 1),
                    Value = r.Next(20, 30),
                    Index = i,
                });
                if (i % 4 == 0)
                    YearData.Add(new Data
                    {
                        Label = "200" + (i / 4),
                        Value = r.Next(35, 45),
                        Index = i,
                        Range = 3,
                    });
            }
            
            DataContext = this;
        }

        public ObservableCollection<Data> QuaterData { get; set; }
        public ObservableCollection<Data> YearData { get; set; }
        
    }

    public class ScatterRangeColumnSeries : ScatterBase
    {
        public ScatterRangeColumnSeries()
        {
            DefaultStyleKey = typeof(ScatterRangeColumnSeries);
        }

        protected override void RenderSeriesOverride(bool animate)
        {
            //base.RenderSeriesOverride(animate);

            if (this.Viewport.IsEmpty || this.RootCanvas == null ||
                    this.XAxis == null || this.YAxis == null ||
                    this.ItemsSource == null)
            {
                return;
            }

            this.RootCanvas.Children.Clear();

           // Dictionary<Point, Point> rectPoints = new Dictionary<Point, Point>();
            List<Rect> rectangles = new List<Rect>();
            double index = 0;
            //Getting Points based on the ItemsSource
            foreach (var item in ItemsSource)
            {
                //Getting X Value
                double xValue = 0;
                object xValObj = item.GetType().GetProperty(XMemberPath).GetValue(item, null);
                if (xValue != null)
                    xValue = Convert.ToDouble(xValObj);

                //Getting Y Value
                double yValue = 0;
                object yValObj = item.GetType().GetProperty(YMemberPath).GetValue(item, null);
                if (yValObj != null)
                    yValue = Convert.ToDouble(yValObj);

                //Gating end range value
                double rValue = 0;
                object rValObj = item.GetType().GetProperty(RangeMemeberPath).GetValue(item, null);
                if (rValObj != null)
                    rValue = Convert.ToDouble(rValObj);

                ScalerParams xParams = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, XAxis.IsInverted);
                ScalerParams yParams = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, YAxis.IsInverted);
                double x = this.XAxis.GetScaledValue(xValue, xParams);
                double y = this.YAxis.GetScaledValue(yValue, yParams);
                double r = this.XAxis.GetScaledValue(xValue + rValue, xParams);

                Point topLeft = new Point(x, y);
                Point bottomRight = new Point(r, RootCanvas.ActualWidth);


                Rect rect = new Rect(topLeft, bottomRight);
                rectangles.Add(rect);
                //rectPoints.Add(topLeft, bottomRight);
            }


            foreach (Rect rect in rectangles)
            {
                Path dataPoint = new Path();
                dataPoint.Fill = this.ActualBrush;
                dataPoint.Stroke = this.Stroke;
                dataPoint.StrokeThickness = this.Thickness;
                dataPoint.Data = new RectangleGeometry(rect);

                RootCanvas.Children.Add(dataPoint);
            }


        }



        public string RangeMemeberPath
        {
            get { return (string)GetValue(RangeMemeberPathProperty); }
            set { SetValue(RangeMemeberPathProperty, value); }
        }

        public static readonly DependencyProperty RangeMemeberPathProperty =
            DependencyProperty.Register("RangeMemeberPath", 
            typeof(string), 
            typeof(ScatterRangeColumnSeries), 
            new UIPropertyMetadata(null));




        public Brush Stroke
        {
            get { return (Brush)GetValue(StrokeProperty); }
            set { SetValue(StrokeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Stroke.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StrokeProperty =
            DependencyProperty.Register("Stroke", typeof(Brush), typeof(ScatterRangeColumnSeries), new UIPropertyMetadata(Brushes.Gray));


    }

    public class IndexToLabelConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            int index = int.Parse(values[0].ToString());
            ObservableCollection<Data> data = values[1] as ObservableCollection<Data>;

            var item = data.Where(i => i.Index == index).FirstOrDefault();

            if (item != null)
            {
                return item.Label;
            }
            return "";
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }


    public class Data
    {
        public string Label { get; set; }
        public double Value { get; set; }
        public int Index { get; set; }
        public int Range { get; set; }
    }
}
